/*
 * 	gpkg.c - Operate on the user package index
 *
 * 	This file is part of Glacier.
 *
 * 	Glacier is free software: you can redistribute it and/or modify it under the terms of the
 * 	GNU General Public License as published by the Free Software Foundation, either
 * 	version 3 of the License, or (at your option) any later version.
 *
 * 	Glacier is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * 	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * 	PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License along with Glacier. If
 * 	not, see <https://www.gnu.org/licenses/>.
 */ 

#include <color.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <glacier_config.h>
#include <glacier_log.h>
#include <glacier_pkgops.h>
#include <glacier_runtime.h>

#include "glacier.h"

static void
usage(int argc, char *argv[])
{
	printf(COL_BLUE "[*] " COL_RESET "%s - operate on the user package index\n", argv[0]);
	printf(COL_BLUE "[?] " COL_RESET "usage: %s [-h] [-v] [-f] [-u] [-x]\n", argv[0]);
	printf("\n");
	printf("%s {-h}		Show this message\n", argv[0]);
	printf("%s {-v} 		Show the current version\n", argv[0]);
	printf("%s {-f} PKG		Install package PKG\n", argv[0]);
	printf("%s {-u} PKG		Update package PKG\n", argv[0]);
	printf("%s {-x} PKG		Remove package PKG\n", argv[0]);
	printf("\n");
	printf("Glacier is free software.\n");
	printf("See the GNU GPL version 3 for details.\n");
}

static void
usage_small(int argc, char *argv[])
{
	printf(COL_RED "[x] " COL_RESET "usage: %s [-h] [-v] [-f] [-u] [-x]\n", argv[0]);
}

int
main(int argc, char *argv[])
{
	if (argc < 2) {
		printf(COL_RED "[x] " COL_RESET "%s: not enough arguments (expected 1, got %i)\n", argv[0], argc - 1);
		usage_small(argc, argv);
		exit(1);
	}

	int opt;

	while ((opt = getopt(argc, argv, "hvfux:")) != -1) {
	switch(opt) {
		case 'h':
			usage(argc, argv);
			return 0;
			break;
		case 'v':
			printf(GLACIER_VERSION "\n");
			return 0;
			break;
		case 'f':
			if (argc < 3) { errlog("No package(s) specified."); return 1; }
			if (is_process_root() == 0) {
				errlog("Failed to open package index: permission denied. Are you root?");
				return(EXIT_FAILURE);
			}
			runtime_exists();
			if (errno != 0) { exit(1); }
			init_config();
			load_all_from_config();
			mkworkspace();
			die_config();
			return 0;
			break;
		case 'u':
			if (argc < 3) { errlog("No package(s) specified."); return 1; }
			is_process_root("Failed to open package index: permission denied. Are you root?");
			mkworkspace();
			return 0;
			break;
		case 'x':
			if (argc < 3) { errlog("No package(s) specified."); return 1; }
			is_process_root("Failed to open package index: permission denied. Are you root?");
			mkworkspace();
			return 0;
			break;
		case '?':
			printf(COL_RED "[x] " COL_RESET "%s: Unknown argument '%s'.\n", argv[0], argv[1]);
			usage_small(argc, argv);
			return 1;
			break;
		}
	}
	
	printf(COL_RED "[x] " COL_RESET "%s: Unknown argument '%s'.\n", argv[0], argv[1]);
	usage_small(argc, argv);
	return 1;
}
